# -*- coding: utf-8 -*-
# Generated by Django 1.11.4 on 2017-08-08 05:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('paypal', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='paypaltransaction',
            name='state',
            field=models.CharField(choices=[('created', 'created'), ('approved', 'approved'), ('failed', 'failed')], default='created', max_length=16),
        ),
    ]
